#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2021/2/1 15:20
# @Author : 心蓝
import time
import threading
import traceback
from multiprocessing.pool import ThreadPool
from unittest.signals import registerResult

from .result import TestResult


class TestRunner:
    resultclass = TestResult

    def run(self, ts):
        result = self._make_result()
        registerResult(result)
        start_time = time.perf_counter()
        startTestRun = getattr(result, 'startTestRun', None)
        if startTestRun is not None:
            startTestRun()
        try:
            ts(result)
        finally:
            stopTestRun = getattr(result, 'stopTestRun', None)
            if stopTestRun is not None:
                stopTestRun()

        stop_time = time.perf_counter()
        duration = stop_time - start_time
        result.duration = duration
        return result

    def _make_result(self):
        return self.resultclass()



class Runner:
    def __init__(self):
        self.result = None
        self.marks = None
        self.thread_num = None
        self.pool = None
        self.lock = None
        self.retry = 0

    def run(self, test_suits):
        self.marks = self.settings.get('marks')
        self.thread_num = self.settings.get('thread_num')
        self.retry = int(self.settings.get('retry', 0))

        if self.marks:
            if isinstance(self.marks, str):
                self.marks = self.marks.split(',')

        self.result = {
            'total_test_suit': len(test_suits),
            'total_test_case': 0,
            'test_suit_result': [],
            'fail_test_case': 0,
            'success_test_case': 0,
            'error_test_case': 0,
            'skip_test_case': 0
        }

        if self.thread_num:
            self.pool = ThreadPool(self.thread_num)
            self.lock = threading.Lock()

            total_s_time = time.time()
            for test_suit in test_suits:
                self.pool.apply_async(self.task, args=(test_suit, ))
            self.pool.close()
            self.pool.join()
        else:
            total_s_time = time.time()
            for test_suit in test_suits:
                self.task(test_suit)
        total_e_time = time.time()
        self.result['duration'] = total_e_time - total_s_time
        if self.result['error_test_case']:
            self.result['status'] = -1
        elif self.result['fail_test_case']:
            self.result['status'] = 1
        else:
            self.result['status'] = 0
        return self.result

    def task(self, test_suit):
        suit_result = {
            'suit_name': test_suit.name,
            'test_case_results': [

            ],
        }
        try:
            test_suit.setUpClass()
            for test_case in test_suit.testcases:
                self.__safe_count_result('total_test_case')
                temp = self.__run_case(test_suit, test_case)
                suit_result['test_case_results'].append(temp)
            test_suit.tearDownClass()
        except Exception as e:
            test_suit.logger.exception(e)
            suit_result['exception'] = traceback.format_exc()
        self.result['test_suit_result'].append(suit_result)

    def __run_case(self, test_suit, test_case):

        temp = {
            'title': test_case['title'],
            # 'case': case
        }
        if self.marks:
            case_mark = test_case.get('marks')
            if not case_mark:
                temp['status'] = 'skipped'
                self.__safe_count_result('skip_test_case')
                return temp
            case_mark = case_mark.split(',')
            if not set(case_mark) & set(self.marks):
                temp['status'] = 'skipped'
                self.__safe_count_result('skip_test_case')
                return temp
        for i in range(self.retry+1):
            if i > 0:
                test_suit.logger.warning('用例【{}】开始第{}次重跑'.format(test_case['title'], i))
            try:
                s_time = time.time()
                test_obj = test_suit()
                test_obj.test(test_case)
                e_time = time.time()
            except AssertionError as e:
                temp['status'] = 'failed'
                temp['exception'] = traceback.format_exc()
                # self.__safe_count_result('fail_test_case')
                test_suit.logger.exception(e)
            except Exception as e:
                temp['status'] = 'broken'
                temp['exception'] = traceback.format_exc()
                # self.__safe_count_result('error_test_case')
                test_suit.logger.exception(e)
            else:
                temp['status'] = 'passed'
                temp['duration'] = (e_time - s_time) * 1000
                self.__safe_count_result('success_test_case')
                break
        if i > 0:
            temp['retry_count'] = i
        temp['case'] = test_obj.case
        if temp['status'] == 'failed':
            self.__safe_count_result('fail_test_case')
        elif temp['status'] == 'broken':
            self.__safe_count_result('error_test_case')

        return temp

    def __safe_count_result(self, key):
        if self.lock:
            self.lock.acquire()
            self.result[key] += 1
            self.lock.release()
        else:
            self.result[key] += 1


#!/usr/bin/python
# -*- coding: UTF-8 -*-
# @Time    : 2021/10/21 9:32 下午
# @Author  : xinlan
import os
import sys
from argparse import ArgumentParser
from configparser import ConfigParser
import yaml
from concurrent.futures import ThreadPoolExecutor
from openpyxl import load_workbook
from .runner import TestRunner
from .common.log_handler import get_logger
from .common.db_handler import DB
from .case import BaseCase
from .loader import TestLoader
from .utils import DjangoJSONEncoder


class TestProgram:
    def __init__(self, args=None, test_data=None, url=None):
        self.test_data = test_data
        self.result = None
        self.args = None
        self.parse_args(args)
        self.test_base_dir = None
        self.settings = {}
        self.get_settings()
        self.logger = get_logger(file=self.settings.get('logfile'), debug=self.settings.get('debug', False))
        if self.test_data is None:
            self.test_data = self.get_test_data_from_file()
        else:
            self.__get_test_data()
        self.TestLoader = TestLoader
        self.TestRunner = TestRunner
        if self.settings.get('auto') == 1:
            self.run_tests()
            print('用例总数:{},成功:{}个,跳过:{},失败:{}个,错误:{}个'.format(
                self.result['total'],
                self.result['passed'],
                self.result['skipped'],
                self.result['failed'],
                self.result['broken']
            ))
            exit(self.result['status'])

    def __get_test_data(self):
        if isinstance(self.test_data, dict):
            test_suits = self.test_data.pop('test_suits')
            self.settings.update(self.test_data)
            self.test_data = test_suits

    def compile_result(self):
        result = {
            'total': 0,
            'status': 0,
            'failed': 0,
            'broken': 0,
            'passed': 0,
            'skipped': 0,
            'test_data': [],
            'duration': 0

        }
        for item in self.result:
            temp = item.result()
            result['total'] += temp.testsRun
            result['failed'] += len(temp.failures)
            result['broken'] += len(temp.errors)
            result['passed'] += temp.passed
            result['skipped'] += len(temp.skipped)
            result['duration'] += temp.duration
            result['test_data'].append(
                {'name': temp.name,
                 'result': temp.result
                 }
            )

        if result['broken'] > 0:
            result['status'] = 2
        elif result['failed'] > 0:
            result['status'] = 1
        result['total'] = result['broken'] + result['failed'] + result['passed'] + result['skipped']
        self.result = result

    def run_tests(self):
        self.result = []
        with ThreadPoolExecutor(max_workers=self.settings.get('thread_num', 1)) as executor:
            for ts in self.create_test():
                self.result.append(executor.submit(self.TestRunner().run, ts))
        self.compile_result()

    def create_test(self):
        for item in self.test_data:
            if not item.get('testcases'):
                continue
            self.add_info_to_data(item)
            yield self.TestLoader(data=item, baseclass=BaseCase).loader()

    def add_info_to_data(self, data):
        data['settings'] = self.settings
        data['logger'] = self.logger.getChild(data['name'])
        data['db'] = DB(**self.settings.get('db_config')) if self.settings.get('db_config', None) else None

    def parse_args(self, args):
        if sys.version < '3.9':
            parser = ArgumentParser(description='easytest 命令行参数帮助')
        else:
            parser = ArgumentParser(description='easytest 命令行参数帮助', exit_on_error=False)
        parser.add_argument('file_or_dir', nargs='?', type=str, help='项目路径，或者需要执行的用例文件')
        parser.add_argument('--debug', action="store_true", help='开启日志调试模式，默认为False')
        parser.add_argument('--logfile', type=str, help='日志文件路径')
        parser.add_argument('--marks', type=str, default='', help='运行标记')
        parser.add_argument('--thread_num', type=int, default=0, help='运行启动线程的数量')
        parser.add_argument('--report', type=str, help='测试报告文件路径，按文件后缀生成对应格式的报告')
        parser.add_argument('--auto', type=int, default=1, help='是否自动执行用例，还是要主动运行方法运行，默认为1表示自动运行')
        if args:
            self.args = vars(parser.parse_args(args))
        else:
            self.args = vars(parser.parse_args())

    def get_settings(self):
        ini_file = os.path.join(os.getcwd(), 'easytest.ini')
        if os.path.exists(ini_file):
            self.settings = self.get_ini_config(ini_file)

        for key, value in self.args.items():
            if value:
                self.settings[key] = value
        self.settings['file_or_dir'] = self.args['file_or_dir']

    @staticmethod
    def get_ini_config(ini_file):
        config = ConfigParser()
        try:
            config.read(ini_file)
        except UnicodeDecodeError:
            try:
                config.read(ini_file, encoding='utf-8')
            except UnicodeDecodeError as e:
                raise e
        res = {
        }
        for section in config.sections():
            if section in ['project', 'run']:
                res.update(dict(config[section]))
            else:
                res[section] = dict(config[section])
        if 'run' in config:
            if 'debug' in config['run']:
                res['debug'] = config['run'].getboolean('debug')
            if 'thread_num' in config['run']:
                res['thread_num'] = config['run'].getint('thread_num')
            if 'retry' in config['run']:
                res['retry'] = config['run'].getint('retry')
        if 'db_config' in config and 'port' in config['db_config']:
            res['db_config']['port'] = config['db_config'].getint('port')
        return res

    def get_test_data_file_name(self):
        files = []
        file = self.args['file_or_dir']
        if file is None:
            file = os.getcwd()
            self.test_base_dir = file
        file = os.path.abspath(file)

        if os.path.isfile(file):
            files.append(file)
            self.test_base_dir = os.path.dirname(file)
        elif os.path.isdir(file):
            self.test_base_dir = file
            for root, _, fs in os.walk(file):
                for f in fs:
                    if os.path.basename(f).startswith('~'):
                        continue
                    if f.split('.')[-1] in ['xlsx', 'yaml', 'yml']:
                        files.append(os.path.join(root, f))
        else:
            self.logger.warning('目录/文件：{}不存在'.format(file))
            raise RuntimeError('目录/文件：{}不存在'.format(file))

        return files

    def get_test_data_from_file(self):
        files = self.get_test_data_file_name()

        for file in files:
            if file.split('.')[-1] == 'xlsx':
                for data in self.load_data_from_excel(file):
                    yield data
            else:
                for data in self.load_data_from_yaml(file):
                    yield data

    def load_data_from_excel(self, file):
        """
        从excel文件中获取测试数据
        :param file:
        :return:
        """
        wb = load_workbook(file)
        sheet_names = wb.sheetnames
        name_prefix = self.get_name_prefix(file)

        for sheet_name in sheet_names:
            sheet = wb[sheet_name]
            suit = {
                "name": ':'.join((name_prefix + [sheet_name])),
                'path': file,
                "testcases": []
            }
            row = sheet.max_row
            column = sheet.max_column
            title = {}
            for i in range(1, column + 1):
                title[i] = sheet.cell(row=1, column=i).value

            for j in range(2, row + 1):
                temp = {}
                for i in range(1, column + 1):
                    temp[title[i]] = sheet.cell(row=j, column=i).value
                suit['testcases'].append(temp)
            yield suit

    def load_data_from_yaml(self, file):
        with open(file, 'rb') as f:
            temp = yaml.load(f, yaml.FullLoader)
        name_prefix = self.get_name_prefix(file)

        data = {'name': ':'.join(name_prefix)}
        if 'test' in temp:
            data.update({
                'path': file,
                'testcases': [temp['test']]
            })

        if 'test_suit' in temp:
            data.update({
                'path': file,
                'testcases': temp['test_suit']
            })

        yield data

    def get_name_prefix(self, filename):
        filename = ''.join(filename.split('.')[:-1])
        res = filename.replace(self.test_base_dir, '').strip('\\/')
        if '/' in res:
            res = res.split('/')
        else:
            res = res.split('\\')
        return res

    def generate_report(self):
        if self.settings['report'].split('.')[-1] == 'json':
            with open(self.settings['report'], 'w', encoding='utf-8') as f:
                import json
                json.dump(self.result, f, ensure_ascii=False, cls=DjangoJSONEncoder)
        else:
            print('正在开发中，敬请期待')

    def run(self):
        self.run_tests()

        return self.return_result()

    def return_result(self):
        if self.result is None:
            self.run_tests()
        return {
            'total_test_suit': len(self.result['test_data']),
            'total_test_case': self.result['total'],
            'test_suit_result': self.result['test_data'],
            'fail_test_case': self.result['failed'],
            'success_test_case': self.result['passed'],
            'error_test_case': self.result['broken'],
            'skip_test_case': self.result['skipped'],
            'duration': self.result['duration'],
            'status': self.result['status']
        }


main = TestProgram



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2022/1/20 19:29
# @Author : 心蓝
import unittest


class TestLoader:
    def __init__(self, data, baseclass):
        self.data = data
        self.baseclass = baseclass

    def loader(self):
        funcs = self._get_test_funcs()
        funcs['settings'] = self.data['settings']
        funcs['logger'] = self.data['logger']
        funcs['db'] = self.data['db']
        testcase_class = type(funcs['name'], (self.baseclass, ), funcs)
        return unittest.defaultTestLoader.loadTestsFromTestCase(testcase_class)

    def _get_test_funcs(self):
        funcs = {}
        for index, item in enumerate(self.data['testcases'], 1):
            func_name = 'test_{:03}_{}'.format(index, item['title'])
            if self.is_marked(item):
                funcs[func_name] = self.wrapper(self.baseclass.flow, item)
            else:
                funcs[func_name] = self.skip_wrapper(self.baseclass.flow, item, self.data['settings'].get('marks', ''))
            funcs[func_name].__doc__ = item['title']
        funcs['name'] = self.data['name']
        return funcs

    def is_marked(self, item):
        marks = self.data['settings'].get('marks', '')
        if marks == '':
            return True

        if set(marks.split(',')) & set((item.get('marks') if item.get('marks') else '').split(',')):
            return True
        return False

    def wrapper(self, func, item):
        #@unittest.skip('test')
        def inner(*args, **kwargs):
            func(testcase=item, *args, **kwargs)
        return inner

    def skip_wrapper(self, func, item, marks):
        @unittest.skip('没有匹配标记:{}'.format(marks))
        def inner(*args, **kwargs):
            func(testcase=item, *args, **kwargs)
        return inner



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2020/12/2 20:13
# @Author : 心蓝
import time
import threading
import pymysql
from multiprocessing.pool import ThreadPool


class DB:
    def __init__(self, pool_size=5, max_size=32, **kwargs):
        if pool_size > max_size:
            pool_size = max_size
        self.pool_size = pool_size
        self.max_size = max_size
        self.db_config = kwargs
        self.db_config['autocommit'] = True
        self.pool = []
        self.running_conn_num = 0
        # self.conn = pymysql.connect(**kwargs)
        self.cursor_args = kwargs.pop('cursor_args', {})
        self.lock = threading.Lock()

        if not self.cursor_args:
            self.cursor_args = {}

    def __get_conn(self):
        if self.pool:
            return self.pool.pop()
        else:
            self.lock.acquire()
            if self.running_conn_num < self.pool_size:
                try:
                    conn = pymysql.connect(**self.db_config, connect_timeout=3)
                    self.running_conn_num += 1
                except Exception as e:
                    raise e
                else:
                    return conn
                finally:
                    self.lock.release()

            else:
                self.lock.release()
                time.sleep(0.1)
                return self.__get_conn()

    def __execute(self, action, sql, **kwargs):
        conn = self.__get_conn()
        with conn.cursor(**self.cursor_args) as cursor:
            count = cursor.execute(sql)

            if action == 'exist':
                if cursor.fetchone():
                    res = True
                else:
                    res = False
            elif action == 'get_one':
                res = cursor.fetchone()
            elif action == 'get_one_value':
                res = cursor.fetchone()
                if isinstance(res, tuple):
                    res = res[0]
                elif isinstance(res, dict):
                    for value in res.values():
                        res = value
            elif action == 'get_many':
                res = cursor.fetchmany(**kwargs)
            elif action == 'get_all':
                res = cursor.fetchall()
            elif action == 'count':
                res = count
        self.pool.append(conn)
        return res

    def exist(self, sql):
        """
        查询是否存在数据
        :param sql: 需要执行的sql
        :return:
        """
        return self.__execute(action='exist', sql=sql)

    def get_one(self, sql):
        """
        获取一条查询结果
        :param sql:
        :return:
        """
        return self.__execute(action='get_one', sql=sql)

    def get_one_value(self, sql):
        """
        获取一个值
        :param sql:
        :return:
        """
        return self.__execute(action='get_one_value', sql=sql)

    def get_many(self, sql, size):
        """
        获取指定条数的查询结果
        :param sql:
        :param size: 指定条数 整数
        :return:
        """
        return self.__execute(action='get_many', sql=sql, size=size)

    def get_all(self, sql):
        """
        获取所有的查询结果
        :param sql:
        :return:
        """
        return self.__execute(action='get_all', sql=sql)

    def get_value(self, sql):
        res = self.__execute(action='get_all', sql=sql)
        res = [item[0] for item in res]
        if len(res) == 0:
            return res
        elif len(res) == 1:
            return res[0]
        else:
            return res

    def __del__(self):
        for conn in self.pool:
            conn.close()
# db = DB(**settings.DATABASE_CONFIG)

if __name__ == '__main__':
    db = DB(
    pool_size= 32,
    host='api.lemonban.com',
    user='future',
    password='123456',
    db='futureloan',
    charset='utf8',
    )
    pool = ThreadPool(2)
    sql = 'select * from member order by id desc limit 1'
    res = []

    for i in range(10):
        res.append(pool.apply_async(db.exist, args=(sql, )))
    pool.close()
    pool.join()

    print(db.running_conn_num, 'a')
    print([x.get() for x in res])


    # with db.conn.cursor() as cursor:
    #     cursor.close()
    #     cursor.execute(sql)
    # print(db.exist(sql))

    # print(db.get_one(sql))
    # cursor = db.conn.cursor()
    # try:
    #     cursor.execute(sql)
    # except:
    #     cursor.close()
    # print(cursor.fetchmany(2))
    # cursor.close()
    # print(db.exist(sql))
    # print(db.get_one(sql))
    # print(db.get_one_value(sql))
    # print(db.get_many(sql, 2))
    # print(db.get_all(sql))
    # res = db.exist("select id from member where mobile_phone='157'")
    # print(res)








#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2020/11/16 21:53
# @Author : 心蓝
"""
日志处理器
"""
import logging


class MyLoggerHandler(logging.Handler):
    def __init__(self, container: list):
        super().__init__()
        self.container = container

    def emit(self, record) -> None:
        msg = self.format(record)
        self.container.append(msg + '\n')


def get_logger(name='easytest', file=None, debug=False):
    if debug:
        # 如果开启了调试模式
        fmt = '%(levelname)s %(asctime)s:%(message)s'
        file_level = logging.DEBUG
        console_level = logging.DEBUG
    else:
        fmt = '%(levelname)s %(asctime)s :%(message)s'
        file_level = logging.INFO
        console_level = logging.INFO

    logger = logging.getLogger(name)
    for handler in logger.handlers:
        logger.removeHandler(handler)
    logger.setLevel(logging.DEBUG)  # 设置等级
    # 3. 创建格式化器
    formatter = logging.Formatter(fmt=fmt)
    # 2. 创建日志处理器


    console_handler = logging.StreamHandler()
    console_handler.setLevel(console_level)  # 设置控制台输出日志的等级
    console_handler.setFormatter(formatter)
    # 5. 把日志处理器添加到日志器

    logger.addHandler(console_handler)
    if file:
        file_handler = logging.FileHandler(filename=file, encoding='utf-8')
        file_handler.setLevel(file_level)  # 设置写入文件的日志等级
        # 4. 把格式化器添加到日志处理器上
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
    return logger


# logger = get_logger(debug=True)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2020/11/13 21:49
# @Author : 心蓝
import os
import re
import random
import json

import yaml
from openpyxl import load_workbook

# from common.db_handler import db


def get_data_from_excel(file, sheet_name=None):
    """
    从excel文件中获取测试用例数据
    :param file: excel文件名
    :param sheet_name: 表名
    :return:
    """
    # 1. 加载excel文件
    wb = load_workbook(file)
    # 2. 获取对应的表
    if sheet_name is None:
        ws = wb.active
    else:
        ws = wb[sheet_name]
    # 获取几行几列
    row = ws.max_row
    column = ws.max_column
    # 3. 获取cell中的数据并组织成需要的结构
    # 创建一个空列表用来接收数据
    data = []
    # 先获取标题
    title = {}
    for i in range(1, column+1):
        title[i] = ws.cell(row=1, column=i).value
    # title = {i: ws.cell(row=1, column=i).value for i in range(1, column+1)}
        # print(ws.cell(row=1, column=i).value)
        # print(1, i)
    # print(title)
    # title = {
    #     1: 'title',
    #     2: 'username',
    #     3: 'password',
    #     4: 'expect'
    # }
    # 获取值
    # print(row, column)
    for j in range(2, row+1):
        # 临时变量，表示一行数据
        temp = {}
        for i in range(1, column+1):
            # print(j, i)

            # print(title[i], ws.cell(row=j, column=i).value, end=' ')
            temp[title[i]] = ws.cell(row=j, column=i).value
        # 循环结束的时候i=column 也就是最后一列 title[i]取出expect
        # print(temp)
        # print(temp[title[i]])
        # temp[title[i]] = eval(temp[title[i]])
        # print(temp)
        data.append(temp)

    return data


def load_data_from_excel(file):
    """
    从excel文件中获取测试数据
    :param file:
    :return:
    """
    wb = load_workbook(file)
    sheet_names = wb.sheetnames

    for sheet_name in sheet_names:
        sheet = wb[sheet_name]
        suit = {
            "name": sheet_name,
            'path': file,
            "testcases": []
        }
        row = sheet.max_row
        column = sheet.max_column
        title = {}
        for i in range(1, column + 1):
            title[i] = sheet.cell(row=1, column=i).value

        for j in range(2, row + 1):
            temp = {}
            for i in range(1, column + 1):
                temp[title[i]] = sheet.cell(row=j, column=i).value
            suit['testcases'].append(temp)
        yield suit


def load_data_from_yaml(file):
    with open(file, 'rb') as f:
        temp = yaml.load(f, yaml.FullLoader)
    data = {'name': os.path.split(file)[-1].split('.')[0]}
    if 'test' in temp:
        data.update({
                'path': file,
                'testcases': [temp['test']]
              })

    if 'test_suit' in temp:
        data.update({
                'path': file,
                'testcases': temp['test_suit']
                })

    yield data


def generate_phone_num():
    """
    随机生成手机号码
    :return:
    """
    # 1开头，11位，第二数3456789，后面9位
    # 1开头，11位，第二个数358
    # 写死，避免后面出问题，如果要测手机号码单独测试
    phone = ['158']
    # 剩下的9位
    for i in range(8):
        phone.append(random.choice('0123456789'))
    phone = ''.join(phone)

    return phone


# def phone_is_exist(phone_num):
#     """
#     手机号码是否已注册
#     :param phone_num:
#     :return:
#     """
#     sql = "select id from member where mobile_phone='{}'".format(phone_num)
#     print(sql)
#     if db.exist(sql):
#         return True
#     return False


def replace_args(s, **kwargs):
    """
    替换参数
    :param s: 需要被替换的用例数据
    :param kwargs: 需要替换的参数
    :return: 替换好的用例数据
    """
    for key, value in kwargs.items():
        s = s.replace('#{}#'.format(key), str(value))
    return s


def replace_args_by_re(s, cls):
    """
    通过正则表达式动态替换参数
    :param s: 需要被替换的json字符串
    :param cls: 提供数据的对象
    :return:
    """
    args = re.findall('#(.*?)#', s)
    for arg in args:
        value = getattr(cls, arg, None)
        if value:
            s = s.replace('#{}#'.format(arg), str(value))
    return s


def extract_data(maps, cls, json_res_str):
    """
    提取需要存储的数据
    :param maps: 需要提取的映射 {cls_arg: key}
    :param cls: 测试用例类
    :param json_res_str: json响应字符串
    :return:
    """
    # 去掉干扰字符
    json_res_str = json_res_str.replace(' ', '').replace('\n', '')

    for arg, key in maps.items():
        # 根据key生成正则表达式
        re_strs = [r'"{}":"(.*?)"'.format(key),  # 字符串参数
                   r'"{}":(\d+?\.\d+?),'.format(key),  # 浮点数参数,放在前面
                   r'"{}":(\d+?),'.format(key),  # 数字参数
                   ]
        # 取响应中提取数据，并保存到类中
        for re_str in re_strs:
            value = re.findall(re_str, json_res_str)
            if value:
                setattr(cls, arg, value[0])
                # 找到就跳出
                break


if __name__ == '__main__':
    res = load_data_from_excel('../project.xlsx')
    # res = get_data_from_excel('../testdata/testcases.xlsx', sheet_name='register')
    # print(res[0]['request_data'].replace('#phone#', generate_phone_num()))
    # phone = generate_phone_num()
    # print(phone)
    # phone = generate_phone_num()
    # print(phone)
    # print(res[0]['expect'], type(res[0]['expect']))
    # print(random.random())  # 返回0-1之间的随机浮点数，左闭右开
    # print(random.randint(a, b)) # 返回整数a到b之间的随机整数，闭区间
    # random.choice()
    # 随机获取ls中的一个字母
    # ls = list('abcdefghijk')
    # 随机获取索引
    # ls的索引区间 0-len(ls)-1
    # random_index = random.randint(0, len(ls)-1)
    # print(ls[random.randint(0, len(ls)-1)])
    # print(random.choice(ls))




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2021/2/1 15:20
# @Author : 心蓝
import re

from .common.db_handler import DB
from .common.log_handler import get_logger
from .case import BaseCase


class TestProducer:

    def __init__(self, data):
        self.data = data

    def produce(self):

        project = {
            'name': self.data.get('name', 'easytest'),
            'host': self.data.get('host'),
            'db_config': self.data.get('db_config'),
            'interfaces': self.data.get('interfaces')
        }
        BaseCase.project = project

        BaseCase.logger = self.logger

        db_handler = self.__get_db_handler()

        BaseCase.db = db_handler

        test_suits = []

        for index, test_suit in enumerate(self.data['test_suits']):
            class_name = 'TestSuit{:05}'.format(index)
            _class = type(class_name, (BaseCase, ), test_suit)

            test_suits.append(_class)
        return test_suits

    def __get_db_handler(self):
        """
        连接数据库
        :param :
        :return:
        """
        if self.data.get('db_config'):
            return DB(**self.data['db_config'])





#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2022/1/20 17:53
# @Author : 心蓝
import time
import unittest
from unittest import TestResult as ts


class TestResult(ts):
    """
    测试结果
    """

    def __init__(self, stream=None, descriptions=None, verbosity=None):
        super().__init__(stream=stream, descriptions=None, verbosity=None)
        self.result = []
        self.passed = 0

    def startTest(self, test: unittest.case.TestCase) -> None:
        super().startTest(test)
        self.start_time = time.perf_counter()

    def stopTest(self, test: unittest.case.TestCase) -> None:
        self.end_time = time.perf_counter()

        super().stopTest(test)
        if test.status != 'skipped':
            temp = {'name': test._testMethodDoc,
                 'status': test.status,
                 'duration': self.end_time-self.start_time,
                 'case': test.case,
                 'logs': test.logs,
                 }
            if self.failures and self.failures[-1][0] == test:
                temp['exception'] = self.failures[-1][1]
            self.result.append(
                temp
            )
        self.name = test.name

    def addSuccess(self, test: unittest.case.TestCase) -> None:
        super(TestResult, self).addSuccess(test)
        test.status = 'passed'
        self.passed += 1

    def addFailure(self, test: unittest.case.TestCase, err) -> None:
        super(TestResult, self).addFailure(test, err)
        test.status = 'failed'

    def addError(self, test, err) -> None:
        super(TestResult, self).addError(test, err)
        test.status = 'broken'

    def addSkip(self, test: unittest.case.TestCase, reason: str) -> None:
        super(TestResult, self).addSkip(test, reason)
        test.status = 'skipped'




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2021/2/1 15:21
# @Author : 心蓝
"""Test case implementation"""
import re
import json
from unittest import TestCase

import requests
from jsonpath import jsonpath
from faker import Faker

from .common.log_handler import MyLoggerHandler
# from .common.log_handler import logger


class BaseCase(TestCase):
    name = None   # 功能名称
    db = None
    # logger = logger  # 日志器

    def add_log_handler(self):
        if self.logger:
            for h in self.logger.handlers:
                self.logger.removeHandler(h)
            self.logs = getattr(self, 'logs', [])
            new_handler = MyLoggerHandler(self.logs)
            new_handler.setLevel(self.logger.parent.handlers[0].level)
            new_handler.setFormatter(self.logger.parent.handlers[0].formatter)
            self.logger.addHandler(new_handler)

    @classmethod
    def setUpClass(cls) -> None:
        pass
        # 类前置
        # cls.logger.info('=========={}测试开始============'.format(cls.name))
        # print('==========注册接口测试开始============')

    @classmethod
    def tearDownClass(cls) -> None:
        pass
        # 类后置
        # cls.logger.info('=========={}测试结束============'.format(cls.name))
        # print('==========注册接口测试结束============')

    @classmethod
    def send_http_request(cls, url, method='get', **kwargs) -> requests.Response:
        """
        sent http request
        :param url: 
        :param method: 
        :param kwargs: 
        :return: 
        """
        if getattr(cls, 'session', None) is None:
            cls.session = requests.session()
        method = method.lower()
        return getattr(cls.session, method)(url, **kwargs)

    def flow(self, testcase):
        self.add_log_handler()
        self.case = testcase
        self._test_name = '测试套件【{}】用例【{}】:'.format(self.__class__.name, self._testMethodName)
        self.logger.debug('{}开始测试>>>>>>>>'.format(self._test_name))
        try:
            self.__process_test_data()

            self.send_request()

            self.__assert_res()

            # self.__extract_data_from_response()
            self.logger.info('{}测试成功<<<<<<<<<'.format(self._test_name))
        except Exception as e:
            self.logger.warning('{}测试失败<<<<<<<<<'.format(self._test_name))
            self.logger.exception(e)
            raise e
        finally:
            self.logger.debug('{}测试结束<<<<<<<<<'.format(self._test_name))

    def __process_test_data(self):
        """
        处理测试数据
        :return:
        """
        self.generate_test_data()
        self.__check_case()
        self.__replace_args()
        self.__process_url()
        self.__handle_request_data()

    def generate_test_data(self):
        """
        动态生成测试数据 phone
        :return:
        """
        fake = Faker('zh-cn')
        fake_keys = dir(fake)
        # 子类可以复写实现自定义的生成测试数据
        # 1 生成动态数据并替换
        temp = json.dumps(self.case)
        keys = re.findall(r'\$(\w*?)\$', temp)
        for key in keys:
            if key not in fake_keys:
                self.logger.error('${}$这个动态数据关键字不支持'.format(key))
                raise ValueError('${}$这个动态数据关键字不支持'.format(key))
            value = str(getattr(fake, key)())
            temp = temp.replace('${}$'.format(key), value)
        self.case = json.loads(temp)

    def __check_case(self):
        """
        检查用例数据格式
        :return:
        """
        keys = ['title', 'method', 'url', 'status_code']
        for key in keys:
            if key not in self.case:
                raise ValueError('用例数据错误！必须包含【{}】字段'.format(key))

    def __process_url(self):
        """
        处理url
        :return:
        """
        if self.case['url'].startswith('http'):
            pass
        elif self.case['url'].startswith('/'):
            self.case['url'] = self.settings.get('host') + self.case['url']
        else:
            if self.case['url'] not in self.settings.get('interfaces'):
                self.logger.warning('{}处理url错误：接口名字 \'{}\'不正确'.format(self._test_name, self.case['url']))
                raise ValueError('{}处理url错误：接口名字 \'{}\'不正确'.format(self._test_name, self.case['url']))
            self.case['url'] = self.settings.get('host') + self.settings.get('interfaces')[self.case['url']]

    def __replace_args(self):
        """
        替换参数
        """
        temp = json.dumps(self.case)
        temp = self.replace_args_by_re(temp)
        self.case = json.loads(temp)

    def __handle_request_data(self):
        """
        处理请求数据
        :return:
        """
        if self.case.get('request'):
            self.case['request'] = self.__loads_by_json_or_eval(self.case['request'])
        else:
            self.case['request'] = {}

    def send_request(self):
        """
        发送请求，当有特殊处理时可以复写
        """
        try:
            self.response = self.send_http_request(url=self.case['url'], method=self.case['method'],
                                                   **self.case['request'])
        except Exception as e:
            self.logger.warning('{}发送http请求错误: {}'.format(self._test_name, e))
            self.logger.warning('url:{}'.format(self.case['url']))
            self.logger.warning('method:{}'.format(self.case['method']))
            self.logger.warning('args:{}'.format(self.case['request']))
            raise RuntimeError('{}发送http请求错误:{}'.format(self._test_name, e))

    def __assert_res(self):
        """
        断言
        """
        self.__assert_status_code()
        self.__assert_response()
        self.__extract_data_from_response()
        self.__assert_db()

    def __assert_status_code(self):
        """
        响应状态码断言
        :return:
        """
        try:
            self.assertEqual(self.case['status_code'], self.response.status_code)
        except AssertionError as e:
            self.logger.warning('{}状态码断言失败！'.format(self._test_name))
            try:
                self.response.text
            except:
                pass
            else:
                self.logger.warning('响应数据为:{}'.format(self.response.text))
            raise e
        else:
            self.logger.debug('{}状态码断言成功！'.format(self._test_name))
            self.get_response_data()

    def get_response_data(self):
        if self.case.get('res_type') is None:
            self.logger.debug('注意：没有设置res_type，只有响应体为空的时候有用')
            return
        if self.case['res_type'].lower() == 'json':
            self.response_data = self.response.json()
        elif self.case['res_type'].lower() == 'xml':
            self.response_data = self.response.text
        elif self.case['res_type'].lower() == 'html':
            self.response_data = self.response.text
        else:
            raise ValueError('{}响应数据提取错误：请指定合适的响应类型,支持的类型有json,xml,html'.format(self._test_name))

    @property
    def assert_funcs(self):
        return {
                    'eq': self.assertEqual,
                    'neq': self.assertNotEqual,
                    'gt': self.assertGreater,
                    'gte': self.assertGreaterEqual,
                    'lt': self.assertLess,
                    'lte': self.assertLessEqual,
                    'in': self.assertIn,
                    'nin': self.assertNotIn
                }

    def __assert_response(self):
        if self.case.get('assertion'):
            self.__check_assertion_field()
            for item in self.case['assertion']:
                actual_res = self.__extract_data_by_express(item[2])
                expect_res = item[0]
                assert_func = self.assert_funcs.get(item[1])
                if assert_func is None:
                    self.logger.error('不支持操作符【{}】'.format(item[1]))
                    raise ValueError('不支持操作符【{}】'.format(item[1]))
                try:
                    assert_func(expect_res, actual_res)
                except Exception as e:
                    self.logger.warning('{}响应数据断言失败'.format(self._test_name))
                    self.logger.warning('请求数据是: {}'.format(self.case['request']))
                    self.logger.warning('期望结果是：{}'.format(expect_res))
                    self.logger.warning('实际结果是：{}'.format(actual_res))
                    self.logger.warning('校验条件是: {}'.format(item))
                    self.logger.warning('响应回的数据是：{}'.format(self.response_data))
                    raise e
            else:
                self.logger.debug('{}响应数据断言成功'.format(self._test_name))

    def __assert_db(self):
        """
        数据库断言
        :return:
        """
        if not self.case.get('db_assertion'):
            return
        if self.db is None:
            raise RuntimeError('没有数据库链接信息，要数据库断言请填写数据库信息')
        self.case['db_assertion'] = self.replace_args_by_re(self.case['db_assertion'])
        self.case['db_assertion'] = self.__loads_by_json_or_eval(self.case['db_assertion'])
        self.__check_db_assertion_field()

        for item in self.case['db_assertion']:
            self.logger.debug('执行sql是: {}'.format(item[2]))
            actual_res = self.db.get_value(item[2])
            expect_res = item[0]

            assert_func = self.assert_funcs.get(item[1])
            if assert_func is None:
                self.logger.error('不支持操作符【{}】'.format(item[1]))
                raise ValueError('不支持操作符【{}】'.format(item[1]))
            try:
                assert_func(expect_res, actual_res)
            except Exception as e:
                self.logger.warning('{}数据库断言失败'.format(self._test_name))
                self.logger.warning('执行sql是: {}'.format(item[1]))
                self.logger.warning('期望结果是：{}'.format(expect_res))
                self.logger.warning('实际结果是：{}'.format(actual_res))
                self.logger.warning('校验条件时: {}'.format(item))
                raise e
        else:
            self.logger.debug('{}数据库断言成功'.format(self._test_name))

    def __loads_by_json_or_eval(self, s):
        """
        将json字符串load为python字典，支持简单的算术表达式，通过eval实现
        :param s:
        :return:
        """
        try:
            if not isinstance(s, str):
                s = json.dumps(s)
            res = json.loads(s)
        except Exception as e:
            try:
                from decimal import Decimal
                res = eval(s)
            except Exception as e:
                self.logger.warning('{}转换json参数失败：{}'.format(self._test_name, e))
                raise ValueError('{}转换json参数失败：{}'.format(self._test_name, e))
            else:
                return res

        else:
            return res

    def replace_args_by_re(self, s):
        """
        通过正则表达式动态替换参数
        :param s: 需要被替换的json字符串
        :return:
        """
        args = re.findall('#(.*?)#', s)
        for arg in set(args):
            value = getattr(self, arg, None)
            if value:
                s = s.replace('#{}#'.format(arg), str(value))
        return s

    def __get_value_from_db(self, condition, sql):
        """
        获取sql对应的值
        :param item:
        :return:
        """
        if condition == 'exist':
            res = self.db.exist(sql)
        elif condition == 'eq':
            res = self.db.get_one_value(sql)
        else:
            raise ValueError('{}数据库断言条件不支持，当前支持：exist,eq'.format(self._test_name))
        return res

    def __extract_data_by_express(self, extract_exp):
        """
        通过表达式提取数据
        """
        if extract_exp.startswith('$'):
            return self.__extract_data_by_jsonpath(extract_exp)
        elif extract_exp.startswith('/'):
            return self.__extract_data_by_xpath(extract_exp)
        else:
            return self.__extract_data_by_re(extract_exp)

    def __extract_data_by_jsonpath(self, extract_exp):
        """
        jsonpath提取数据
        :param obj:
        :param extract_exp:
        :return:
        """
        if self.case.get('res_type') is None or self.case.get('res_type').lower() != 'json':
            res = jsonpath(dict(self.response.headers), extract_exp)
            if res is False:
                raise ValueError('提取请求头的jsonpath表达式错误')
        else:
            res = jsonpath(self.response.json(), extract_exp)
            if res is False:
                res = jsonpath(dict(self.response.headers), extract_exp)
        if res:
            if len(res) == 1:
                return res[0]
            return res

        raise ValueError('{}jsonpath表达式:{}错误'.format(self._test_name, extract_exp))

    def __extract_data_by_xpath(self, extract_exp):
        pass

    def __extract_data_by_re(self, extract_exp):
        res = re.findall(extract_exp, self.response.text)
        if res:
            if len(res) == 1:
                res = res[0]
            return res
        else:
            raise ValueError('正则表达式错误，提取不到值！')

    @staticmethod
    def __extract_data_from_json(obj, extract_exp):
        """
        从json数据里提取数据
        """
        res = jsonpath(obj, extract_exp)
        if not res:
            res = re.findall(extract_exp, json.dumps(obj, ensure_ascii=False))
            if not res:
                return '没有匹配到值'
                # raise ValueError('用例【{}】断言提取表达式:{} 错误'.format(self.case['title'], extract_exp))
        else:
            if len(res) == 1:
                res = res[0]
        return res

    def __extract_actual_res(self, extract_exp):
        if self.case.get('res_type') is None:
            raise ValueError('没有设置res_type')
        if self.case['res_type'].lower() == 'json':
            res = self.response.json()

        if extract_exp[0] == '.':
            keys = extract_exp[1:].split('.')
            for key in keys:
                res = res.get(key)
        return res

    def __extract_data_from_response(self):
        """
        从响应中提取数据
        :return:
        """
        if self.case.get('extract'):

            self.__check_extract_field()
            for item in self.case['extract']:
                value = self.__extract_data_by_express(item[1])
                setattr(self.__class__, item[0], value)

    def __check_extract_field(self):
        """
        检查extract字段格式
        :return:
        """
        self.__check_field('extract', 2)

    def __check_assertion_field(self):
        """
        检查assertion字段格式
        :return:
        """
        self.__check_field('assertion', 3)

    def __check_db_assertion_field(self):
        """
        检查db_assertion字段格式
        :return:
        """
        if isinstance(self.case['db_assertion'], list):
            for item in self.case['db_assertion']:
                if not len(item) == 3:
                    raise ValueError(
                        '{}{}字段格式错误: {} 不是一个{}元列表'.format(self._test_name, 'db_assertion', 3, item))

        else:
            raise ValueError('{0}{1}字段格式错误:{1}字段应该是一个列表'.format(self._test_name, 'db_assertion'))

    def __check_field(self, field_name, length):
        """
        检查field_name字段格式
        :param field_name:
        :param length
        :return:
        """
        try:
            self.case[field_name] = self.__loads_by_json_or_eval(self.case[field_name])
        except Exception as e:
            self.logger.error('{}{}字段json格式错误: {}'.format(self._test_name, field_name, e))
            raise ValueError('{}{}字段json格式错误: {}'.format(self._test_name, field_name, e))

        if isinstance(self.case[field_name], list):
            for item in self.case[field_name]:
                if not len(item) == length:
                    raise ValueError('{}{}字段格式错误: {} 不是一个{}元列表'.format(self._test_name, field_name, item, length))

        else:
            raise ValueError('{0}{1}字段格式错误:{1}字段应该是一个列表'.format(self._test_name, field_name))







#!/usr/bin/python
# -*- coding: UTF-8 -*-
# @Time    : 2021/10/29 2:42 下午
# @Author  : xinlan
import json
import datetime
import uuid
import decimal


def is_aware(value):
    """
    Determine if a given datetime.datetime is aware.

    The concept is defined in Python's docs:
    https://docs.python.org/library/datetime.html#datetime.tzinfo

    Assuming value.tzinfo is either None or a proper datetime.tzinfo,
    value.utcoffset() implements the appropriate logic.
    """
    return value.utcoffset() is not None

def _get_duration_components(duration):
    days = duration.days
    seconds = duration.seconds
    microseconds = duration.microseconds

    minutes = seconds // 60
    seconds = seconds % 60

    hours = minutes // 60
    minutes = minutes % 60

    return days, hours, minutes, seconds, microseconds


def duration_string(duration):
    """Version of str(timedelta) which is not English specific."""
    days, hours, minutes, seconds, microseconds = _get_duration_components(duration)

    string = '{:02d}:{:02d}:{:02d}'.format(hours, minutes, seconds)
    if days:
        string = '{} '.format(days) + string
    if microseconds:
        string += '.{:06d}'.format(microseconds)

    return string


def duration_iso_string(duration):
    if duration < datetime.timedelta(0):
        sign = '-'
        duration *= -1
    else:
        sign = ''

    days, hours, minutes, seconds, microseconds = _get_duration_components(duration)
    ms = '.{:06d}'.format(microseconds) if microseconds else ""
    return '{}P{}DT{:02d}H{:02d}M{:02d}{}S'.format(sign, days, hours, minutes, seconds, ms)


def duration_microseconds(delta):
    return (24 * 60 * 60 * delta.days + delta.seconds) * 1000000 + delta.microseconds


class DjangoJSONEncoder(json.JSONEncoder):
    """
    JSONEncoder subclass that knows how to encode date/time, decimal types, and
    UUIDs.
    """
    def default(self, o):
        # See "Date Time String Format" in the ECMA-262 specification.
        if isinstance(o, datetime.datetime):
            r = o.isoformat()
            if o.microsecond:
                r = r[:23] + r[26:]
            if r.endswith('+00:00'):
                r = r[:-6] + 'Z'
            return r
        elif isinstance(o, datetime.date):
            return o.isoformat()
        elif isinstance(o, datetime.time):
            if is_aware(o):
                raise ValueError("JSON can't represent timezone-aware times.")
            r = o.isoformat()
            if o.microsecond:
                r = r[:12]
            return r
        elif isinstance(o, datetime.timedelta):
            return duration_iso_string(o)
        elif isinstance(o, (decimal.Decimal, uuid.UUID)):
            return str(o)
        else:
            return super().default(o)